/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2011-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */

#ifndef headerfile_winix_plugins_export_exportinfo
#define headerfile_winix_plugins_export_exportinfo

#include "core/system.h"
#include "db/db.h"
#include "export.h"
#include "edb.h"
#include "message.h"
#include "exportthread.h"



namespace Export
{


class ExportInfo
{
public:

	ExportInfo();

	void SetSystem(System * psystem);
	void SetConfig(Config * pconfig);
	void SetEDb(EDb * pdb);
	void SetDb(Db * pdb);
	void SetExportThread(ExportThread * pexport_thread);

	void ReadExportDirs();
	void ReadConfigVars();

	ExportDir * FindDir(long dir_id);

	bool DecodePass(Export & exp);

	void SendFile(const Item & item, bool thumb = false);
	void SendDir(const Item & item);
	void SendDir(long dir_id);
	void SendAllFilesFromDir(long dir_id);

	void ResetRecurrenceCheck();

private:

	System * system;
	Config * config;
	EDb * edb;
	Db * db;
	ExportThread * export_thread;

	bool use_rsa;
	std::wstring rsa_key;
	std::string pass_decrypted;
	Message msg;
	Export exp;
	std::wstring tmp_dir;
	std::vector<ExportDir> export_dirs;
	DbItemQuery iq_dir;
	std::vector<Item> dir_items;
	std::vector<std::wstring> additional_export;
	std::vector<Item*> additional_export_dir_tab;
	Item additional_export_item;
	std::vector<long> recurrence_check;

	bool SkipDir(long dir_id, std::wstring & dir);

	void AdditionalExport(const Item & item);
	void AdditionalExport(const Item & item, PT::Space & meta);
	void AdditionalExport(const std::wstring & path);

	bool HasRecurrenceId(long id);
};

}

#endif

