/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "templates.h"
#include "misc.h"
#include "core/request.h"
#include "core/misc.h"


namespace TemplatesFunctions
{


void stat_item_type_is_file(Info & i)
{
	i.res = cur->request->is_item && cur->request->item.file_type == WINIX_ITEM_FILETYPE_NONE;
}



void stat_item_type_is_static_file(Info & i)
{
	i.res = cur->request->is_item && cur->request->item.file_type != WINIX_ITEM_FILETYPE_NONE;
}



void stat_item_type_is_dir(Info & i)
{
	i.res = !cur->request->is_item;
}



void stat_item_inode(Info & i)
{
	i.out << cur->request->last_item->id;
}


void stat_item_user(Info & i)
{
	User * puser = system->users.GetUser(cur->request->last_item->user_id);
	print_user_name(i, puser, cur->request->last_item->guest_name);
}


void stat_item_group(Info & i)
{
	long group_id = cur->request->last_item->group_id;
	Group * pgroup = system->groups.GetGroup(group_id);
	
	if( pgroup )
		i.out << pgroup->name;
	else
		i.out << group_id;
}


void stat_item_privileges(Info & i)
{
	i.out << Toa(cur->request->last_item->privileges, 8);
}


void stat_item_date_creation(Info & i)
{
	tm rtm = system->LocalTime(cur->request->last_item->date_creation);
	i.out << DateToStr(rtm.tm_year + 1900, rtm.tm_mon + 1, rtm.tm_mday, rtm.tm_hour, rtm.tm_min, rtm.tm_sec);
}


void stat_item_date_modification(Info & i)
{
	tm rtm = system->LocalTime(cur->request->last_item->date_modification);
	i.out << DateToStr(rtm.tm_year + 1900, rtm.tm_mon + 1, rtm.tm_mday, rtm.tm_hour, rtm.tm_min, rtm.tm_sec);
}




void stat_item_template(Info & i)
{
	i.out << cur->request->last_item->html_template;
}



void stat_item_is_template_from_mount_point(Info & i)
{
	i.res = cur->request->last_item->html_template.empty();
}


} // namespace TemplatesFunctions



