/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_templates_templates
#define headerfile_winix_templates_templates

#include <iomanip>
#include <set>
#include "ezc.h"
#include "misc.h"
#include "patterncacher.h"
#include "indexpatterns.h"
#include "locale.h"
#include "patterns.h"
#include "changepatterns.h"
#include "htmltextstream.h"
#include "localefilter.h"
#include "core/config.h"
#include "core/cur.h"
#include "core/system.h"
#include "core/sessionmanager.h"
#include "core/htmlfilter.h"
#include "db/db.h"



class Functions;



namespace TemplatesFunctions
{
	extern size_t pat_index;
	extern size_t pat_index_fullscreen;
	extern size_t pat_index_rawcontent;
	extern size_t pat_err_404;
	extern size_t pat_err_per_denied;

	extern Patterns          patterns;
	extern IndexPatterns     index_patterns;
	extern ChangePatterns    change_patterns;
	extern PatternCacher     pattern_cacher;
	extern Locale            locale;
	extern EzcFun            ezc_functions;
	extern LocaleFilter      locale_filter;
	extern HTMLFilter        html_filter;
	extern Ezc::Stack        empty_stack;

	extern Db      * db;
	extern Cur     * cur;
	extern Config  * config;
	extern System  * system;
	extern Functions * functions;
	extern SessionManager * session_manager;





	/*
		adduser
	*/
	void adduser_last_login(Info & i);
	void adduser_last_email(Info & i);


	/*
		config
	*/
	void config_use_ssl(Info & i);
	void config_use_ssl_static(Info & i);
	void config_use_ssl_common(Info & i);
	void config_url_proto(Info & i);
	void config_url_ssl_proto(Info & i);
	void config_base_url(Info & i);
	void config_base_url_static(Info & i);
	void config_base_url_common(Info & i);


	/*
		dir
	*/
	void dir(Info & i);
	void dir_without_slash(Info & i);
	void dir_is_root(Info & i);
	void dir_parent(Info & i);
	void dir_parent_without_slash(Info & i);
	void dir_can_read_exec(Info & i);
	void dir_can_write(Info & i);
	void dir_can_remove(Info & i);
	void dir_can_use_emacs(Info & i);
	void dir_can_use_mkdir(Info & i);
	void dir_is(Info & i);
	void dir_has_parents(Info & i);

	void dir_childs_tab(Info & i);
	void dir_childs_is_parent(Info & i);
	void dir_childs_tab_url(Info & i);
	void dir_childs_tab_privileges(Info & i);
	void dir_childs_tab_user(Info & i);
	void dir_childs_tab_group(Info & i);

	void dir_tab(Info & i);
	void dir_tab_url(Info & i);
	void dir_tab_link(Info & i);

	void dir_last_link_to(Info &);
	void dir_last_is_link_redirect(Info & i);
	void dir_last_subject(Info & i);
	void dir_last_user(Info & i);
	void dir_last_url(Info & i);
	void dir_last_url_is(Info & i);
	void dir_last_date_creation(Info & i);
	void dir_last_date_modification(Info & i);
	void dir_last_date_creation_nice(Info & i);
	void dir_last_date_modification_nice(Info & i);
	void dir_last_dates_equal(Info & i);
	void dir_last_users_different(Info & i);
	void dir_last_modification_user(Info & i);
	void dir_last_html_template(Info & i);
	void dir_last_has_html_template(Info & i);
	void dir_last_meta_str(Info & i);


	/*
		doc
	*/
	void doc_title(Info & i);
	void doc_proto(Info & i);
	void doc_proto_static(Info & i);
	void doc_proto_common(Info & i);
	void doc_base_url(Info & i);
	void doc_base_url_static(Info & i);
	void doc_base_url_common(Info & i);
	void doc_current_url(Info & i);
	void doc_css_tab(Info & i);
	void doc_css_tab_file(Info & i);
	void doc_css_tab_file_is_global(Info & i);
	void doc_css_tab_has_next(Info & i);
	void doc_css_is_empty(Info & i);
	void doc_css_is_one(Info & i);
	void doc_css_more_than_one(Info & i);


	/*
		env
	*/
	void env_user_admin_env_str(Info & i);
	void env_user_env_str(Info & i);
	void env_user_id(Info & i);
	void env_user_name(Info & i);
	void env_user_tab(Info & i);
	void env_user_tab_id(Info & i);
	void env_user_tab_name(Info & i);
	void env_user_tab_is_current(Info & i);


	/*
		filters
	*/
	void fil_urlencode(Info & i);
	void fil_qencode(Info & i);
	void fil_capitalize(Info & i);
	void fil_tosmall(Info & i);
	void fil_firstup(Info & i);
	void fil_first_wordup(Info & i);


	/*
		insert
	*/
	void insert_page(Info & i);


	/*
		item
	*/
	void item_is(Info & i); // !! change to is_item lub na item_defined
	void item_id(Info & i);
	void item_subject(Info & i);
	void item_subject_noescape(Info & i);
	void item_content(Info & i);
	void item_content_noescape(Info & i);
	void item_content_type_is(Item & item, Info & i);
	void item_content_type_is(Info & i);
	void item_content_is_empty(Info & i);
	void item_print_content(HtmlTextStream & out, const std::wstring & content, Item::ContentType content_type);
	void item_print_content(Info & i);
	void item_privileges(Info & i);
	void item_dir(Info & i);
	void item_url(Info & i);
	void item_url_is(Info & i);
	void item_link(Info & i);
	void item_filetype_is_none(Info & i);
	void item_filetype_is_image(Info & i);
	void item_has_static_file(Info & i);
	void item_has_thumb(Info & i);
	void item_can_read(Info & i);
	void item_can_write(Info & i);
	void item_can_remove(Info & i);
	void item_user(Info & i);
	void item_modification_user(Info & i);
	void item_users_different(Info & i);
	void item_date_creation(Info & i);
	void item_date_modification(Info & i);
	void item_date_creation_nice(Info & i);
	void item_date_modification_nice(Info & i);
	void item_dates_equal(Info & i);
	void item_run(Info & i);
	void item_guest_name(Info & i);
	void item_html_template(Info & i);
	void item_has_html_template(Info & i);
	void item_type_is_dir(Info & i);
	void item_type_is_file(Info & i);
	void item_type_is_symlink(Info & i);
	void item_is_link_to(Info & i);
	void item_link_to(Info & i);
	void item_is_link_redirect(Info & i);
	void item_file_size(Info & i);
	void item_sort(Info & i);
	void item_meta_str(Info & i);
	void item_meta(Info & i);
	void item_meta_tab(Info & i);
	void item_meta_tab_value(Info & i);
	void item_meta_tab_has_next(Info & i);

	void item_tab(Info & i);
	void item_tab_index(Info & i);
	void item_tab_id(Info & i);
	void item_tab_subject(Info & i);
	void item_tab_subject_noescape(Info & i);
	void item_tab_content(Info & i);
	void item_tab_content_noescape(Info & i);
	void item_tab_print_content(Info & i);
	void item_tab_privileges(Info & i);
	void item_tab_dir(Info & i);
	void item_tab_url(Info & i);
	void item_tab_link(Info & i);
	void item_tab_filetype_is_none(Info & i);
	void item_tab_filetype_is_image(Info & i);
	void item_tab_can_read(Info & i);
	void item_tab_can_write(Info & i);
	void item_tab_user(Info & i);
	void item_tab_modification_user(Info & i);
	void item_tab_users_different(Info & i);
	void item_tab_group(Info & i);
	void item_tab_date_creation(Info & i);
	void item_tab_date_modification(Info & i);
	void item_tab_date_creation_nice(Info & i);
	void item_tab_date_modification_nice(Info & i);
	void item_tab_dates_equal(Info & i);
	void item_tab_run(Info & i);
	void item_tab_can_use_emacs(Info & i);
	void item_tab_has_static_file(Info & i);
	void item_tab_has_thumb(Info & i);
	void item_tab_type_is_dir(Info & i);
	void item_tab_type_is_file(Info & i);
	void item_tab_type_is_symlink(Info & i);
	void item_tab_is_link_to(Info & i);
	void item_tab_link_to(Info & i);
	void item_tab_is_link_redirect(Info & i);
	void item_tab_file_size(Info & i);
	void item_tab_sort(Info & i);
	void item_tab_has_next(Info & i);


	/*
		last
	*/
	void last_tab(Info & i);
	void last_tab_name(Info & i);
	void last_tab_ip(Info & i);
	void last_tab_start(Info & i);
	void last_tab_end(Info & i);


	/*
		login
	*/
	void login_path(Info & i);


	/*
		man
	*/
	void man_winixfun_tab(Info & i);
	void man_winixfun_tab_index(Info & i);
	void man_winixfun_tab_name(Info & i);
	void man_ezcfun_tab(Info & i);
	void man_ezcfun_tab_index(Info & i);
	void man_ezcfun_tab_name(Info & i);


	/*
		mount
	*/
	void mount_type_is(Info & i);
	void mount_page_arg_is(Info & i);
	void mount_lang_arg(Info & i);
	void mount_lang_arg_is(Info & i);
	void mount_has_html_template(Info & i);
	void mount_first_html_template(Info & i);
	void mount_css_tab(Info & i);
	void mount_css_tab_file(Info & i);
	void mount_css_tab_file_is_global(Info & i);
	void mount_css_tab_has_next(Info & i);
	void mount_css_is_empty(Info & i);
	void mount_css_is_one(Info & i);
	void mount_css_more_than_one(Info & i);
	void mount_cur_type(Info & i);
	void mount_cur_dir(Info & i);
	void mount_cur_fs(Info & i);
	void mount_cur_parlist(Info & i);
	void mount_tab(Info & i);
	void mount_tab_type(Info & i);
	void mount_tab_dir(Info & i);
	void mount_tab_fs(Info & i);
	void mount_tab_parlist(Info & i);


	/*

	*/
	void ls_ckeditor_funnum_browse(Info & i);


	/*
		privileges
	*/
	void priv_user_tab(Info & i);
	void priv_user_tab_name(Info & i);
	void priv_user_tab_isdefault(Info & i);
	
	void priv_group_tab(Info & i);
	void priv_group_tab_name(Info & i);
	void priv_group_tab_isdefault(Info & i);
	
	void priv_privileges(Info & i);
	void priv_privileges_for_files(Info & i);
	void priv_privileges_for_dirs(Info & i);
	
	void priv_show_form_chown(Info & i);
	void priv_show_form_chmod(Info & i);


	/*
		pw
	*/
	void pw_users_change_password_code(Info & i);
	void pw_change_password_login(Info & i);


	/*
		rebus
	*/
	void rebus_question(Info & i);


	/*
		slog
	*/
	void slog_tab(Info & i);
	void slog_tab_is_info(Info & i);
	void slog_tab_is_warning(Info & i);
	void slog_tab_is_error(Info & i);
	void slog_tab_print(Info & i);


	/*
		stat
	*/
	void stat_item_type_is_file(Info & i);
	void stat_item_type_is_static_file(Info & i);
	void stat_item_type_is_dir(Info & i);
	void stat_item_inode(Info & i);
	void stat_item_user(Info & i);
	void stat_item_group(Info & i);
	void stat_item_privileges(Info & i);
	void stat_item_date_creation(Info & i);
	void stat_item_date_modification(Info & i);
	void stat_item_template(Info & i);
	void stat_item_is_template_from_mount_point(Info & i);


	/*
		sys
	*/
	void sys_ver_major(Info & i);
	void sys_ver_minor(Info & i);
	void sys_ver_revision(Info & i);
	void sys_plugin_tab(Info & i);
	void sys_plugin_tab_has_name(Info & i);
	void sys_plugin_tab_name(Info & i);


	/*
		upload
	*/
	void upload_ckeditor_funnum(Info & i);


	/*
		uptime
	*/
	void uptime_more_than_one_day(Info & i);
	void uptime_days(Info & i);
	void uptime_hours(Info & i);


	/*
		user
	*/
	void user_id(Info & i);
	void user_name(Info & i);
	void user_logged(Info & i);
	void user_super_user(Info & i);
	void user_is_in_group(Info & i);
	void user_is_in_all_groups(Info & i);
	void user_can_use_html(Info & i);
	void user_can_use_bbcode(Info & i);
	void user_can_use_raw(Info & i);
	void user_tab(Info & i);
	void user_tab_index(Info & i);
	void user_tab_id(Info & i);
	void user_tab_name(Info & i);
	void user_tab_is_super_user(Info & i);
	void user_tab_is_active(Info & i);
	void user_tab_is_suspended(Info & i);
	void user_tab_is_blocked(Info & i);
	void user_tab_is_current(Info & i);


	/*
		template (html templates)
	*/
	void template_index(Info & i);
	void template_tab(Info & i);
	void template_tab_index(Info & i);
	void template_tab_isdefault(Info & i);
	void template_tab_file_name(Info & i);


	/*
		winix
	*/
	void winix_account_need_email_verification(Info & i);
	void winix_cur_time(Info & i);
	void winix_how_many_sessions(Info & i);
	void winix_users_logged(Info & i);
	void winix_function(Info & i);
	void winix_function_is(Info & i);
	void winix_function_param_is(Info & i);
	void winix_function_param_is_not(Info & i);
	void winix_function_param_value(Info & i);
	void winix_function_param_value_is(Info & i);
	void winix_function_param_value_is_not(Info & i);
	void winix_has_plugin(Info & i);
	void winix_loadavg_now(Info & i);
	void winix_loadavg_1(Info & i);
	void winix_loadavg_5(Info & i);
	void winix_loadavg_15(Info & i);
	void winix_req_per_sec_now(Info & i);
	void winix_req_per_sec_1(Info & i);
	void winix_req_per_sec_5(Info & i);
	void winix_req_per_sec_15(Info & i);
	void winix_err_is(Info & i);
	void winix_err_code(Info & i);
	void winix_is_err_in_locales(Info & i);
	void winix_err_msg_from_locales(Info & i);
	void winix_show_content_in_full_window(Info & i);
	void winix_false(Info & i);
	void winix_true(Info & i);
	void winix_has_postvar(Info & i);
	void winix_postvar(Info & i);
	void winix_postvar_value_is(Info & i);
	void winix_postvar_value_is_not(Info & i);
	void str(Info & i);
	void strnc(Info & i);


	/*
		who
	*/
	void who_tab(Info & i);
	void who_tab_lp(Info & i);
	void who_tab_user(Info & i);
	void who_tab_time(Info & i);
	void who_tab_last_time(Info & i);


} // namespace TemplatesFunctions









class Templates
{
public:

	Templates();

	void SetConfig(Config * pconfig);
	void SetCur(Cur * pcur);
	void SetDb(Db * pdb);
	void SetSystem(System * psystem);
	void SetFunctions(Functions * pfunctions);
	void SetSessionManager(SessionManager * psession_manager);

	void Init();
	void RequestEnd();
	void ReadTemplates();
	void ReadNewIndexTemplates();
	void ReadNewChangeTemplates();
	void Generate();
	void GenerateRunRaw();
	void Generate(Ezc::Pattern & pattern);

private:
	
	void ReadTemplatesForFunctions();
	void ReadIndexTemplates();
	void ReadChangeTemplates();
	void ReadLocale();
	void SetHtmlFilter();
	void CreateFunctions(); // should be called before reading patterns (patterns will cache ezc functions)
	Ezc::Pattern * SelectIndexPatternFromItemAndMountPoint();
	Ezc::Pattern * SelectIndexPattern();

	TemplatesFunctions::EzcGen generator;
	std::wstring temp;
	std::wstring fun_file;
};



#endif
